@@ -1,7 +1,7 @@
 from __future__ import unicode_literals
 
 import logging
-import os
+import os, sys, subprocess
 
 import tornado.web
 
@@ -11,18 +11,30 @@ from configobj import ConfigObj, ConfigObjError
 from validate import Validator
 import jinja2
 
-__version__ = '0.1.1'
+__version__ = '0.1.2'
 
 logger = logging.getLogger(__name__)
 
 spec_file = os.path.join(os.path.dirname(__file__), 'settingsspec.ini')
 template_file = os.path.join(os.path.dirname(__file__), 'index.html')
-config_file = '/etc/mopidy/mopidy.conf'
+#config_file = '/etc/mopidy/mopidy.conf'
 
 #log_file = '/var/log/mopidy/mopidy.log'
 
 password_mask = '******'
 
+def restart_program():
+    """
+    DOES NOT WORK WELL WITH MOPIDY
+    Hack from https://www.daniweb.com/software-development/python/code/260268/restart-your-python-program
+    to support updating the settings, since mopidy is not able to do that yet
+    Restarts the current program
+    Note: this function does not return. Any cleanup action (like
+    saving data) must be done before calling this function"""
+
+    python = sys.executable
+    os.execl(python, python, * sys.argv)
+
 class Extension(ext.Extension):
     dist_name = 'Mopidy-WebSettings'
     ext_name = 'websettings'
@@ -47,7 +59,8 @@ class Extension(ext.Extension):
 
 class WebSettingsRequestHandler(tornado.web.RequestHandler):
 
-    def initialize(self, core):
+    def initialize(self, core, config):
+        self.config_file = config.get('websettings')['config_file']
         self.core = core
 
     def get(self):
@@ -55,11 +68,12 @@ class WebSettingsRequestHandler(tornado.web.RequestHandler):
         templateEnv = jinja2.Environment( loader=templateLoader )
         template = templateEnv.get_template(template_file)
         error = ''
+        logger.info(self.config_file)
         #read config file
         try:
-            iniconfig = ConfigObj(config_file, configspec=spec_file, file_error=True, encoding='utf8')
+            iniconfig = ConfigObj(self.config_file, configspec=spec_file, file_error=True, encoding='utf8')
         except (ConfigObjError, IOError), e:
-            error = 'Could not load ini file! %s %s %s', e, ConfigObjError, IOError
+            error = 'Could not load ini file! %s %s %s' % (e, ConfigObjError, IOError)
         #read values of valid items (in the spec-file)
         validItems = ConfigObj(spec_file, encoding='utf8')
         templateVars = {
@@ -79,38 +93,73 @@ class WebSettingsRequestHandler(tornado.web.RequestHandler):
                     pass
         self.write(template.render ( templateVars ) )
 
+class WebPostRequestHandler(tornado.web.RequestHandler):
+
+    def initialize(self, core, config):
+        self.config_file = config.get('websettings')['config_file']
+        self.core = core
+
     def post(self):
         error = ''
         try:
-            iniconfig = ConfigObj(config_file, configspec=spec_file, file_error=True, encoding='utf8')
+            iniconfig = ConfigObj(self.config_file, configspec=spec_file, file_error=True, encoding='utf8')
         except (ConfigObjError, IOError), e:
             error = 'Could not load ini file!'
-        validItems = ConfigObj(spec_file, encoding='utf8')
-        templateVars = {
-            "error": error
-        }
-        #iterate over the items, so that only valid items are processed
-        for item in validItems:
-            for subitem in validItems[item]:
-                itemName = item + '__' + subitem
-                argumentItem = self.get_argument(itemName)
-                if argumentItem:
-                    #don't edit config value if password mask
-                    if subitem[-8:] == 'password':
-                      if argumentItem == password_mask or argumentItem == '':
-                          continue
-                    iniconfig[item][subitem] = argumentItem
-        iniconfig.write()
-        logger.info('Rebooting system')
-        self.write("<html><body><h1>Settings Saved!</h1><script>toast('Applying changes (reboot)...', 5000);setTimeout(function(){window.location.assign('/');}, 10000);</script><a href='/'>Back</a></body></html>")
+        if error == '':
+            validItems = ConfigObj(spec_file, encoding='utf8')
+            templateVars = {
+                "error": error
+            }
+            #iterate over the items, so that only valid items are processed
+            for item in validItems:
+                for subitem in validItems[ item ]:
+                    itemName = item + '__' + subitem
+                    argumentItem = self.get_argument(itemName, default='')
+                    if argumentItem:
+                        #don't edit config value if password mask
+                        if subitem[-8:] == 'password':
+                          if argumentItem == password_mask or argumentItem == '':
+                              continue
+                        iniconfig[item][subitem] = argumentItem
+            iniconfig.write()
+            error = 'Settings Saved!'
+        message = '<html><body><h1>' + error + '</h1><p>Applying changes (reboot) <br/><a href="/">Back</a><br/></p></body></html>'
+        self.write(message)
 
-        #using two different methods for reboot
+        #logger.info ("restart")
+        #restart_program()
+
+        #using two different methods for reboot for different systems
+        logger.info('Rebooting system')
         os.system("sudo shutdown -r now")
         os.system("shutdown -r now")
         #        os.system("/opt/musicbox/startup.sh")
 
+class WebRebootRequestHandler(tornado.web.RequestHandler):
+
+    def initialize(self, core):
+        self.core = core
+
+    def post(self):
+        logger.info('Halting system')
+        os.system("sudo shutdown -r now")
+        os.system("shutdown -r now")
+
+class WebShutdownRequestHandler(tornado.web.RequestHandler):
+
+    def initialize(self, core):
+        self.core = core
+
+    def post(self):
+        logger.info('Halting system')
+        os.system("sudo shutdown -h now")
+        os.system("shutdown -h now")
+
+
 def websettings_app_factory(config, core):
-    config_file = config.get('websettings', 'config_file')
     return [
-        ('/', WebSettingsRequestHandler, {'core': core})
+        ('/', WebSettingsRequestHandler, {'core': core, 'config': config}),
+        ('/update', WebPostRequestHandler, {'core': core, 'config': config}),
+        ('/reboot', WebRebootRequestHandler, {'core': core}),
+        ('/shutdown', WebShutdownRequestHandler, {'core': core})
     ]
